from panda3d.core import (Vec2, GeomVertexData, GeomVertexFormat, Geom, GeomTriangles, GeomVertexWriter, 
GeomNode, Texture, TextureAttrib, NodePath, RenderState, Material, MaterialAttrib)

CHUNK_SIZE = 2
MAX_VIEW_DIST = 50

vdata = GeomVertexData('name', GeomVertexFormat.getV3n3t2(), Geom.UHStatic)
vdata.setNumRows(3)

vertex = GeomVertexWriter(vdata, 'vertex')
normal = GeomVertexWriter(vdata, 'normal')
texcoord = GeomVertexWriter(vdata, 'texcoord')

vertex.addData3(-1, -1, 0)
vertex.addData3(1, -1, 0)
vertex.addData3(1, 1, 0)
vertex.addData3(-1, 1, 0)

normal.addData3(0, 0, 1)
normal.addData3(0, 0, 1)
normal.addData3(0, 0, 1)
normal.addData3(0, 0, 1)

texcoord.addData2(0, 0)
texcoord.addData2(1, 0)
texcoord.addData2(1, 1)
texcoord.addData2(0, 1)

prim = GeomTriangles(Geom.UHStatic)
prim.addVertices(0, 1, 2)
prim.addVertices(0, 2, 3)
prim.closePrimitive()

blank_geom = Geom(vdata)
blank_geom.addPrimitive(prim)

blank_mat = Material("Material1")
blank_mat.set_shininess(15.0)
blank_mat.set_base_color((0, 1, 0, 1))

class Chunk:
    def __init__(self, tiles_pos, data_chunk):
        self.position = tiles_pos * CHUNK_SIZE

        state = RenderState.make(MaterialAttrib.make(blank_mat))

        geom_node = GeomNode('tile:{}'.format(tiles_pos))
        geom_node.add_geom(blank_geom, state)

        self.model = NodePath(geom_node)
        self.model.hide()
        self.model.reparent_to(render)
        self.model.set_pos(self.position.x, self.position.y, 0)
        self.model.set_color(data_chunk.color)

    def update(self, spectator):
        if spectator.get_distance(self.model) <= MAX_VIEW_DIST:
            if self.model.is_hidden():
                self.model.show()
        else:
            if not self.model.is_hidden():
                self.model.hide()

class Terrain:
    def __init__(self):
        self.view_dst = round(MAX_VIEW_DIST/CHUNK_SIZE)
        self.spectator = NodePath("None")
        self.storage_сhunk = {}
        self.terrain_сhunk = {}

    def add_сhunk_storage(self, pos, geom):
        self.storage_сhunk[pos] = geom

    def update(self):
        spectator_coord_x = round(self.spectator.get_x()/CHUNK_SIZE)
        spectator_coord_y = round(self.spectator.get_y()/CHUNK_SIZE)

        yOffset = -self.view_dst
        while yOffset <= self.view_dst:
            xOffset = -self.view_dst
            while xOffset <= self.view_dst:
                tiles_pos = Vec2(spectator_coord_x + xOffset, spectator_coord_y + yOffset)
                if tiles_pos in self.storage_сhunk:
                    if tiles_pos in self.terrain_сhunk:
                        self.terrain_сhunk[tiles_pos].update(self.spectator)
                    else:
                        self.terrain_сhunk[tiles_pos] = Chunk(tiles_pos, self.storage_сhunk[tiles_pos])
                xOffset += 1
            yOffset += 1

        for сhunk_сoord, сhunk in list(self.terrain_сhunk.items()):
            vec = Vec2(self.spectator.get_x(), self.spectator.get_y()) - сhunk.position
            if vec.length_squared() > 50000:
                self.terrain_сhunk.pop(сhunk_сoord)
                сhunk.model.remove_node()
                #print("unload: ", сhunk)